package com.caimei365.commodity.components;


import com.caimei365.commodity.mapper.PriceMapper;
import com.caimei365.commodity.mapper.PromotionsMapper;
import com.caimei365.commodity.model.vo.*;
import com.caimei365.commodity.utils.MathUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;
import java.util.stream.IntStream;

/**
 * Description
 *
 * @author : Charles
 * @date : 2020/6/22
 */
@Slf4j
@Service
public class PriceUtilService {
    @Resource
    private PriceMapper priceMapper;
    @Resource
    private PromotionsMapper promotionsMapper;
    /**
     * 根据价格返回价格等级
     * @param price 价格
     * @return priceGrade 价格等级
     */
    public Integer getPriceGrade(Double price){
        int grade = 1;
        if (price > 50 * 10000) {
            grade = 5;
        } else if (price > 25 * 10000) {
            grade = 4;
        } else if (price > 10 * 10000) {
            grade = 3;
        } else if (price > 2.5 * 10000) {
            grade = 2;
        }
        return grade;
    }

    /**
     * 设置阶梯价展示数据
     * @param ladderPrices  List<LadderPriceVo>
     * @param tax           TaxVo
     */
    public void setLadderPriceList(List<LadderPriceVo> ladderPrices, TaxVo tax) {
        IntStream.range(0, ladderPrices.size()).forEach(i -> {
            //税费标志
            boolean taxFlag = "0".equals(tax.getIncludedTax()) && ("1".equals(tax.getInvoiceType()) || "2".equals(tax.getInvoiceType()));
            //添加税费
            if (taxFlag) {
                BigDecimal taxFee = MathUtil.div(MathUtil.mul(ladderPrices.get(i).getBuyPrice(), tax.getTaxPoint()), 100);
                ladderPrices.get(i).setBuyPrice(MathUtil.add(ladderPrices.get(i).getBuyPrice(),taxFee).doubleValue());
            }
            if (i == ladderPrices.size() - 1) {
                ladderPrices.get(i).setMaxNum(0);
                ladderPrices.get(i).setNumRange("≥" + ladderPrices.get(i).getBuyNum());
            } else {
                ladderPrices.get(i).setMaxNum(ladderPrices.get(i + 1).getBuyNum());
                String buyNumRangeShow = ladderPrices.get(i).getBuyNum() + "~" + (ladderPrices.get(i + 1).getBuyNum() - 1);
                ladderPrices.get(i).setNumRange(buyNumRangeShow);
            }
        });
    }

    /**
     * 根据用户id设置详细价格
     *
     * @param price 商品价格类
     * @param userId 用户Id
     */
    public void setPriceByUserId(PriceVo price, Integer userId) {
        // 根据用户Id查询用户身份
        Integer identity = priceMapper.getIdentityByUserId(userId);
        if (null != identity && identity > 0) {
            // 用户身份: 2-会员机构, 4-普通机构
            price.setUserIdentity(identity);
            // 默认非促销活动状态
            price.setActStatus(0);
            // 设置划线价
            price.setOriginalPrice(price.getPrice());
            //税费标志
            boolean taxFlag = "0".equals(price.getIncludedTax()) && ("1".equals(price.getInvoiceType()) || "2".equals(price.getInvoiceType()));
            // 根据商品id查询商品活动
            PromotionsVo promotions = promotionsMapper.getPromotionsByProductId(price.getProductId());
            if (null != promotions) {
                // 促销活动
                price.setActStatus(1);
                price.setLadderPriceFlag(0);
                if (promotions.getMode() == 3) {
                    // 获取赠品
                    List<ProductItemVo> giftList = promotionsMapper.getPromotionGifts(promotions.getId());
                    promotions.setGiftList(giftList);
                }
                if (promotions.getType() == 1 && promotions.getMode() == 1 && null != promotions.getTouchPrice()) {
                    price.setPrice(promotions.getTouchPrice());
                    //添加税费
                    if (taxFlag) {
                        BigDecimal taxFee = MathUtil.div(MathUtil.mul(promotions.getTouchPrice(), price.getTaxRate()), 100);
                        promotions.setTouchPrice(MathUtil.add(promotions.getTouchPrice(),taxFee).doubleValue());
                    }
                }
                price.setPromotions(promotions);
            } else {
                if (null != userId) {
                    if (1 == price.getLadderPriceFlag()) {
                        // 阶梯价
                        LadderPriceVo lowerPrice = priceMapper.findLowerLadderPrice(price.getProductId());
                        LadderPriceVo lowerBuyNum = priceMapper.findMaxLadderPrice(price.getProductId());
                        if (null != lowerPrice) {
                            price.setPrice(lowerPrice.getBuyPrice());
                            if (null != lowerBuyNum){
                                price.setMinBuyNumber(lowerBuyNum.getBuyNum());
                            }
                        } else {
                            price.setLadderPriceFlag(0);
                        }
                    } else {
                        // 复购价
                        Double repurchase = priceMapper.getRepurchasePrice(price.getProductId(), userId);
                        if (null != repurchase && repurchase > 0) {
                            price.setPrice(repurchase);
                            price.setLadderPriceFlag(0);
                        }
                    }
                }
            }
            //添加税费
            if (taxFlag) {
                BigDecimal thisTaxFee = MathUtil.div(MathUtil.mul(price.getPrice(), price.getTaxRate()), 100);
                price.setPrice(MathUtil.add(price.getPrice(), thisTaxFee).doubleValue());
            }
        } else {
            price.setPrice(0d);
            price.setOriginalPrice(0d);
            price.setUserIdentity(0);
        }
        // 屏蔽成本价
        price.setCostProportional(0d);
        price.setCostPrice(0d);
    }

        /**
     * 根据用户id设置详细价格
     *
     * @param product 商品价格类
     * @param userId 用户Id
     */
    public void setItemPriceByUserId(ProductItemVo product, Integer userId) {
        // 设置价格等级
        product.setPriceGrade(getPriceGrade(product.getPrice()));
        // 根据用户Id查询用户身份
        Integer identity = priceMapper.getIdentityByUserId(userId);
        if (null != identity && identity > 0) {
            // 用户身份: 2-会员机构, 4-普通机构
            product.setUserIdentity(identity);
            // 默认非促销活动状态
            product.setActStatus(0);
            // 设置划线价
            product.setOriginalPrice(product.getPrice());
            //税费标志
            boolean taxFlag = "0".equals(product.getIncludedTax()) && ("1".equals(product.getInvoiceType()) || "2".equals(product.getInvoiceType()));
            // 根据商品id查询商品活动
            PromotionsVo promotions = promotionsMapper.getPromotionsByProductId(product.getProductId());
            if (null != promotions) {
                // 促销活动
                product.setActStatus(1);
                product.setLadderFlag(0);
                if (promotions.getMode() == 3) {
                    // 获取赠品
                    List<ProductItemVo> giftList = promotionsMapper.getPromotionGifts(promotions.getId());
                    promotions.setGiftList(giftList);
                }
                if (promotions.getType() == 1 && promotions.getMode() == 1 && null != promotions.getTouchPrice()) {
                    product.setPrice(promotions.getTouchPrice());
                    //添加税费
                    if (taxFlag) {
                        BigDecimal taxFee = MathUtil.div(MathUtil.mul(promotions.getTouchPrice(), product.getTaxRate()), 100);
                        promotions.setTouchPrice(MathUtil.add(promotions.getTouchPrice(),taxFee).doubleValue());
                    }
                }
                product.setPromotions(promotions);
            } else {
                if (null != userId) {
                    if (1 == product.getLadderFlag()) {
                        // 阶梯价
                        LadderPriceVo lowerPrice = priceMapper.findLowerLadderPrice(product.getProductId());
                        LadderPriceVo lowerBuyNum = priceMapper.findMaxLadderPrice(product.getProductId());
                        if (null != lowerPrice) {
                            product.setPrice(lowerPrice.getBuyPrice());
                            if (null != lowerBuyNum){
                                product.setMin(lowerBuyNum.getBuyNum());
                            }
                        } else {
                            product.setLadderFlag(0);
                        }
                    } else {
                        // 复购价
                        Double repurchase = priceMapper.getRepurchasePrice(product.getProductId(), userId);
                        if (null != repurchase && repurchase > 0) {
                            product.setPrice(repurchase);
                            product.setLadderFlag(0);
                        }
                    }
                }
            }
            //添加税费
            if (taxFlag) {
                BigDecimal thisTaxFee = MathUtil.div(MathUtil.mul(product.getPrice(), product.getTaxRate()), 100);
                product.setPrice(MathUtil.add(product.getPrice(), thisTaxFee).doubleValue());
            }
        } else {
            product.setPrice(0d);
            product.setOriginalPrice(0d);
            product.setUserIdentity(0);
        }
    }
}
