package com.caimei365.commodity.controller;

import com.caimei365.commodity.model.ResponseJson;
import com.caimei365.commodity.model.vo.BigTypeVo;
import com.caimei365.commodity.model.vo.BrandVo;
import com.caimei365.commodity.service.PageService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/4/12
 */
@Api(tags="商品页面数据API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/commodity")
public class ProductPageApi {

    private final PageService pageService;
    /**
     * 获取分类列表
     *
     * @param typeSort 分类类型：1产品，2仪器
     * @param source   请求来源：www,crm
     */
    @ApiOperation("获取分类菜单（旧：/product/classify）")
    @ApiImplicitParams({
        @ApiImplicitParam(required = true, name = "typeSort", value = "分类类型：1产品，2仪器"),
        @ApiImplicitParam(required = false, name = "source", value = "请求来源：www,crm")
    })
    @GetMapping("/classify")
    public ResponseJson<List<BigTypeVo>> getClassify(String typeSort, String source) {
        return pageService.getClassify(typeSort, source);
    }

    /**
     * 产品/仪器页面数据
     *
     * @param pageId 页面信息id
     * @param userId 用户id
     * @param source 来源：1网站，2小程序
     */
    @ApiOperation("产品/仪器页面数据（旧：/home/instrumentData）")
    @ApiImplicitParams({
        @ApiImplicitParam(required = true, name = "pageId", value = "页面信息id"),
        @ApiImplicitParam(required = false, name = "userId", value = "用户id"),
        @ApiImplicitParam(required = false, name = "source", value = "来源：1网站，2小程序")
    })
    @GetMapping("/classify/product")
    public ResponseJson<Map<String, Object>> getClassifyData(Integer pageId, Integer userId, Integer source) {
        if (pageId == null) {
            return ResponseJson.error("参数异常", null);
        }
        return pageService.getClassifyData(pageId, userId, source);
    }

    /**
     * 首页楼层数据
     *
     * @param userId 用户id
     * @param source 来源 : 1 网站 ; 2 小程序
     */
    @ApiOperation("首页楼层数据(旧：/home/data)")
    @ApiImplicitParams({
        @ApiImplicitParam(required = false, name = "userId", value = "用户id"),
        @ApiImplicitParam(required = false, name = "source", value = "来源：1网站，2小程序")
    })
    @GetMapping("/home/floor")
    public ResponseJson<Map<String, Object>> getHomeData(Integer userId, Integer source) {
        return pageService.getHomeData(userId, source);
    }

    /**
     * 商品品牌列表
     */
    @GetMapping("brandList")
    public ResponseJson<List<BrandVo>> getBrandList() {
        return pageService.getBrandList();
    }
}
