package com.caimei365.commodity.controller;

import com.caimei365.commodity.model.ResponseJson;
import com.caimei365.commodity.model.search.ProductListVo;
import com.caimei365.commodity.model.vo.ProductShopVO;
import com.caimei365.commodity.service.ShopService;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/4/16
 */
@Api(tags="供应商商品API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/commodity/shop")
public class ProductShopApi {
    private final ShopService shopService;

    /**
     * 供应商-主推商品
     */
    @ApiOperation("供应商-主推商品（旧：/supplier/home/products）")
    @ApiImplicitParams({
        @ApiImplicitParam(required = true, name = "shopId", value = "供应商Id（旧：supplierId）"),
        @ApiImplicitParam(required = false, name = "identity", value = "用户身份：1协销，2会员机构，3供应商，4普通机构")
    })
    @GetMapping("/home/products")
    public ResponseJson<List<ProductListVo>> getSupplierMainProducts(Integer shopId, @RequestParam(value = "identity", defaultValue = "0") Integer identity) {
        return shopService.getMainProducts(shopId, identity);
    }

    /**
     * 供应商-我的商品列表
     */
    @ApiOperation("供应商-我的商品（旧：/supplier/myProduct）")
    @ApiImplicitParams({
        @ApiImplicitParam(required = true, name = "shopId", value = "供应商Id"),
        @ApiImplicitParam(required = false, name = "name", value = "商品名称"),
        @ApiImplicitParam(required = false, name = "productCode", value = "货号"),
        @ApiImplicitParam(required = false, name = "validFlag", value = "状态：0逻辑删除，1待审核，2已上架，3已下架，8审核未通过，9已隐身，10已冻结"),
        @ApiImplicitParam(required = false, name = "featuredFlag", value = "是否主推：0否，1是"),
        @ApiImplicitParam(required = false, name = "typeSort", value = "分类类型：1产品，2仪器"),
        @ApiImplicitParam(required = false, name = "bigTypeId", value = "一级分类Id"),
        @ApiImplicitParam(required = false, name = "smallTypeId", value = "二级分类Id"),
        @ApiImplicitParam(required = false, name = "tinyTypeId", value = "三级级分类Id"),
        @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
        @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("/products")
    public ResponseJson<PageInfo<ProductShopVO>> getShopProducts(Integer shopId, String name, String productCode,
       Integer validFlag, Integer featuredFlag, Integer typeSort, Integer bigTypeId, Integer smallTypeId, Integer tinyTypeId,
                                             @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                             @RequestParam(value = "pageSize", defaultValue = "20") int pageSize) {
        return shopService.getShopProducts(shopId, name, productCode, validFlag, featuredFlag, typeSort, bigTypeId, smallTypeId, tinyTypeId, pageNum, pageSize);
    }

}
