package com.caimei365.commodity.mapper;

import com.caimei365.commodity.model.vo.*;
import org.apache.ibatis.annotations.Mapper;

import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/4/12
 */
@Mapper
public interface PageMapper {
    /**
     * 获取大分类
     * @param typeSort 分类类型：1产品，2仪器
     * @param source   请求来源：www,crm
     */
    List<BigTypeVo> getBigTypeList(String typeSort, String source);
    /**
     * 根据大分类Id获取二级分类
     * @param bigTypeId 大分类Id
     * @param source   请求来源：www,crm
     */
    List<SmallTypeVo> getSmallTypeList(Integer bigTypeId, String source);
    /**
     * 根据二级分类Id获取三级分类
     * @param smallTypeId 二级分类Id
     * @param source      请求来源：www,crm
     */
    List<TinyTypeVo> getTinyTypeList(Integer smallTypeId, String source);
    /**
     * 查询页面信息
     * @param pageId 页面Id
     */
    Integer getPageTypeSort(Integer pageId);
    /**
     * 查询分页详情热搜词
     * @param pageId
     */
    List<HotSearchVo> getHotSearchByPageId(Integer pageId, Integer source);
    /**
     * 首页楼层
     * @param source
     */
    List<PageFloorVo> getHomePageFloor(Integer source);
    /**
     * 查询页面信息商品楼层
     * @param pageId
     */
    List<PageFloorVo> getFloorByPageId(Integer pageId, Integer source);
    /**
     * 查询楼层内容
     * @param id 页面信息楼层id
     */
    FloorContentVo getFloorContentById(Integer id);
    /**
     * 查询楼层相关图片
     * @param id 页面信息楼层id
     * @param source
     */
    List<FloorImageVo> getFloorImageById(Integer id, Integer source);
    /**
     * 获取商品及价格
     * @param productId
     */
    ProductItemVo getProductItemById(Integer productId);
    /**
     * 优质供应商楼层
     */
    ShopFloorVo getSupplierFloorImage();
    /**
     * 查询优质供应商数据
     * @param source
     */
    List<ShopImageVo> getSupplierImage(Integer source);
    /**
     * 商品品牌列表
     */
    List<BrandVo> getBrandList();
}
